import java.net.*;
import java.util.*;
import java.io.*;
import java.lang.*;
import java.applet.Applet;
import java.util.StringTokenizer;

/** A Simple Cipher 5, written by Dan Kiser, 2000
 *  <p>
 *  This class is used to encode and decode a text
 *  string using a randomly generated key. The key 
 *  can contain 12 to 24 characters.  An exclusive OR 
 *  is performed with the key on the text to encode  
 *  and decode the text string.
 *
 */

public class cipher5 {

  private StringBuffer sbHost;  // Holds IP address of the host.
  private String strIP;         // Holds printable IP of host.

  //*****************************************************************
  // Class constructor takes as input a reference to the applet.
  //
  public cipher5(Applet app) {

    sbHost = new StringBuffer(); 

    try { 

      URL base = new URL(app.getDocumentBase(), "/");

      InetAddress[] ipaddr = InetAddress.getAllByName(base.getHost());

      // Get highest of IP addresses.
      int a = 0;

      byte[] ip1 = ipaddr[0].getAddress();

//      System.out.println("IP 0 = " + ipaddr[0].toString());

      for (int i=1; i < ipaddr.length; i++) {

        byte[] ip2 = ipaddr[i].getAddress();

//        System.out.println("IP " + i + " = " + ipaddr[i].toString());

        if (ip1.length == ip2.length) {
          int j=0;

          while (ip1[j] == ip2[j] && j < ip1.length) j++;

          if (j < ip1.length && ip1[j] < ip2[j]) { a = i; ip1 = ip2; }
        }

      }

      StringTokenizer st = new StringTokenizer(ipaddr[a].getHostAddress(),".");

//      System.out.println("IP token count = " + st.countTokens());

      while(st.hasMoreTokens()) {
        Integer c = new Integer(st.nextToken());
        sbHost.append((char) c.intValue());
      }
      
      strIP = ipaddr[a].toString();

    }
    catch (Exception e) { 
      // No IP address.  Use address 0.0.0.0
      strIP = "IP not found";
      for(int i=0; i < 4; i++) sbHost.append('0');
      System.err.println("IP not found: " + e);
    }

  }


  //*****************************************************************
  // Class constructor takes as input the host IP address string.
  //
  public cipher5(String ipaddr) {

    sbHost = new StringBuffer(ipaddr);

  }


  //*****************************************************************
  // Generate a random number between 12 and 24.  The minimum key
  // length is 12 and the maximum key length is 24.
  //
  private int getKeySize() {

    int s = Math.round((float)Math.abs(Math.floor(Math.random() * 25)));

    return( (s > 12) ? s : s+12 );

  } // getKeySize


  //*****************************************************************
  // Generate a random number between 1 and 65535.  
  //
  private int setKey() {

    int k = Math.round((float)Math.abs(Math.floor(Math.random() * 65535)));

    return( k+1 );

  } // setKey


  //*****************************************************************
  // This method sets a key used to encode/decode the input string.
  //
  private StringBuffer getKey() {

    int size = getKeySize();

    StringBuffer key = new StringBuffer();

    for(int i=0; i < size; i++) {

        key.append( (char)setKey() );

    }

    return key;

  }  // getKey


  //*****************************************************************
  // This method is used to convert a string buffer to a string of 
  // hex characters (0-9,A-F).
  //
  static public String encodeHex(StringBuffer input) {

    char[] tmp = new char[input.length()];
    String hex = new String();
    String pad = "0000";

    input.getChars(0,input.length(),tmp,0);

    for (int i = 0; i < tmp.length; i++) {

      // Get hex value of character
      String num = Integer.toHexString( (int)tmp[i] );

      // Pad with leading zeros
      hex += pad.substring(0,4-num.length()) + num;

    }

    return( hex );

  } // encodeHex


  //*****************************************************************
  // This method is used to convert a string of hex characters to a
  // string buffer of characters.
  //
  static public StringBuffer decodeHex(String hex) {

    StringBuffer tmp = new StringBuffer();

    for (int i = 0; i < hex.length(); i+=4) {

      try {
          tmp.append((char)Integer.parseInt( hex.substring(i,i+4), 16 ));
      }
      catch( NumberFormatException e ) { }

    }

    return( tmp );

  } // decodeHex


  //*****************************************************************
  // This method performs a simple cipher alogrithm to convert the 
  // input string to cipher/plain text. Input includes the string and
  // the key used to encode/decode the string.  Each character in the
  // input string is XOR'ed with a corresponding character in the
  // key.
  //
  private StringBuffer code(StringBuffer key, StringBuffer txt) {

    StringBuffer out = new StringBuffer();

    // Convert key to array of chars
    char[] keyChars = new char[key.length()];
    key.getChars(0,key.length(),keyChars,0);
    int klen = keyChars.length;

    // Convert plain text to array of chars
    char[] txtChars = new char[txt.length()];
    txt.getChars(0,txt.length(),txtChars,0);
    int clen = txtChars.length;

    for(int i=0; i < txtChars.length; i++) {

      // XOR char and append to output buffer
      out.append((char)(txtChars[i] ^ keyChars[i%klen]));

    }

    return( out );

  } // code


  //*****************************************************************
  // The method converts plain text into an encoded string. A key
  // generated to encode the string is appended to the beginning 
  // of the output string.  A hex encoded string is returned.
  //
  public String encode(StringBuffer sbTxt) {

    StringBuffer sbKey = getKey();

    StringBuffer buff1 = code(sbKey, sbTxt);

    String len = (new Integer(sbKey.length())).toString();

    if (len.length() < 2) len = "0" + len;

    StringBuffer buff2 = new StringBuffer(len);

    for (int i=0; i < sbKey.length(); i++) buff2.append(sbKey.charAt(i));
    for (int i=0; i < buff1.length(); i++) buff2.append(buff1.charAt(i));

    StringBuffer buff3 = code(sbHost, buff2);

    sbKey.setLength(0);
    buff1.setLength(0);
    buff2.setLength(0);

    String out = encodeHex(buff3);

    buff3.setLength(0);

    return( out );

  } // encode


  //*****************************************************************
  // The method converts plain text into an encoded string. The 
  // key argument is used to encode the string. The hex encoded
  // string is returned.
  //
  public String encode(String txt, String key) {

    StringBuffer sbTxt = new StringBuffer(txt);
    StringBuffer sbKey = new StringBuffer(key);

    StringBuffer buff1 = code(sbKey, sbTxt);

    sbTxt.setLength(0);
    sbKey.setLength(0);

    String out = encodeHex(buff1);

    buff1.setLength(0);

    return( out );

  } // encode


  //*****************************************************************
  // The method converts an encoded string into plain text. The 
  // key is extracted from the beginning of the string.  A string 
  // buffer is returned.
  //
  public StringBuffer decode(String txt) {

    StringBuffer sbTxt = code(sbHost, decodeHex(txt));

    StringBuffer len = new StringBuffer();

    len.append(sbTxt.charAt(0));
    len.append(sbTxt.charAt(1));

    int cnt = (new Integer(len.toString())).intValue();    

    StringBuffer sbKey = new StringBuffer();

    int i = 2;

    for (; i < cnt+2; i++) sbKey.append(sbTxt.charAt(i));

    StringBuffer buff1 = new StringBuffer();

    for (; i < sbTxt.length(); i++) buff1.append(sbTxt.charAt(i));

    StringBuffer out = code(sbKey, buff1);

    sbTxt.setLength(0);
    sbKey.setLength(0);
    buff1.setLength(0);

    return( out );

  } // decode


  //*****************************************************************
  // The method converts an encoded string into plain text using
  // the key.  A string buffer is returned.
  //
  public StringBuffer decode(String txt, String key) {

    StringBuffer sbTxt = decodeHex(txt);

    StringBuffer sbKey = new StringBuffer(key);

    StringBuffer out = code(sbKey, sbTxt);

    sbTxt.setLength(0);
    sbKey.setLength(0);

    return( out );

  } // decode


  //*****************************************************************
  // The method returns the IP address used by the cipher.
  //
  public String getIP() {

    return sbHost.toString();

  } // getIP


  //*****************************************************************
  // The method prints the IP address used by the cipher.
  //
  public void printIP() {

    System.out.println("IP Address = " + strIP);

  } // printIP


} // cipher5
