import java.lang.*;
import java.io.*;


/** fetcher class, written by Dan Kiser, 1999
 *  <p>
 *  This class is used to retrieve a document. The document
 *  is parsed to extract the secret links.  The secret links
 *  are stored in a vector created by the password class.
 *
 */
public class fetcher extends Thread {

  private final int SIZE = 21;     // String length of script code

  protected password app;          // Applet reference
  protected String url;            // Encoded URL to fetch
  protected cipher5 c5;            // Cipher5 object

  //*****************************************************************
  // Class constructor takes as input a reference to the applet and
  // the encoded URL string.
  //
  public fetcher() {

    super("fetcher");

  } // fetcher


  //*******************************************************
  // The run method is overriden in the subclass.
  // 
  public void run() {


  } // run


  //*******************************************************
  // This method is used to check if the given URL is
  // absolute. An absolute URL has a colon char indicating
  // the presence of the protocol.
  // 
  protected boolean absolute(StringBuffer sbUrl) {

    boolean match = false;

    for (int i=0; i < sbUrl.length(); i++) {
      if (sbUrl.charAt(i) == ':') match = true;
    }

    return match;

  } // absolute


  //*******************************************************
  // This method is used to append the user name to the
  // URL's query string. 
  // 
  protected StringBuffer addUser(StringBuffer sbUrl) {

    boolean isQuery = false;

    for (int i=0; i < sbUrl.length(); i++) {
      if (sbUrl.charAt(i) == '?') isQuery = true;
    }
  
    if (isQuery) {
      sbUrl.append("&user=");
      sbUrl.append(app.user);
    }
    else {
      sbUrl.append("?user=");
      sbUrl.append(app.user);
    }

    return sbUrl;

  } // addUser
  

  //*******************************************************
  // The readTag method is called when a < is encountered
  // in the input stream.  This method is responsible
  // for reading the remainder of the tag.
  // Note that when this method has been called the <
  // has been read from the input stream but has not yet 
  // been sent to the output stream. 
  //
  // This method has trouble (as do most web browsers) 
  // if it encounters a raw < sign in the Stream. Technically
  // raw < signs should be encoded as &lt; in the original HTML.
  //
  protected StringBuffer readTag(DataInputStream is) {
  
    StringBuffer theTag = new StringBuffer("<");
    char theChar = '<';
  
    try {
       while (theChar != '>') {
         theChar = (char) is.readByte();
         theTag.append(theChar);
       } 
     }
     catch (EOFException e) {
       // Done with the Stream
     }
     catch (Exception e) {
        System.err.println(e);
     }     

     return( theTag );
  
  } // readTag


  //*******************************************************
  // The checkTag method takes a complete tag as a string
  // and check to see if it represents a link. If it is a 
  // link for a secret page, the link is saved to the URL
  // vector. The actual link is replaced in the tag by its
  // index into this vector.
  //
  protected String checkTag(StringBuffer sb) {
  
    // temporary position variables
    int p1, p2, p3;

    p1 = -1;

    // remove newlines from tag
    for (int i=0; i < sb.length(); i++) {
        if (sb.charAt(i) == '\r' || sb.charAt(i) == '\n') {
            sb.setCharAt(i,' ');
        }
    }


    try {

      // HTML tags are cases insensitive so converting
      // it to upper case makes the problem slightly easier
      char c[] = new char[SIZE];
      sb.getChars(0,7,c,0);
      String s1 =  new String(c).toUpperCase();

      // Anchor tag?
      if (s1.startsWith("<A ")) {

        int i = 3;
        boolean js = false;

        while (i < sb.length() && !js) {

          if (sb.charAt(i) == 'P' || sb.charAt(i) == 'p') {
            sb.getChars(i,i+SIZE,c,0);
            s1 = new String(c).toUpperCase();
            if (s1.startsWith("PARENT.NAVPANE.FETCH(")) {
              p1 = i; js = true;
            }
          }

          i++;

        } // while

      }

      // Form tag?
      else if (s1.startsWith("<FORM ")) {

        int i = 6;
        boolean js = false;

        while (i < sb.length() && !js) {

          if (sb.charAt(i) == 'P' || sb.charAt(i) == 'p') {
            sb.getChars(i,i+SIZE,c,0);
            s1 = new String(c).toUpperCase();
            if (s1.startsWith("PARENT.NAVPANE.QUERY(")) {
              p1 = i; js = true;
            }
          }

          i++;

        } // while
        
      }

      // Title tag?
      else if (s1.startsWith("<TITLE>")) {
        sb = sb.insert(0,"<META HTTP-EQUIV=\"Pragma\" CONTENT=\"no-cache\">\n"); 
        return sb.toString();
      }

      // This is not a link based tag
      else { 
        return sb.toString();      
      }

      // Return if link does not contain a call to fetch
      if (p1 == -1) return sb.toString();

      p2 = p1 + SIZE;

      while (Character.isWhitespace(sb.charAt(p2))) p2++;

      if (sb.charAt(p2) == '\'') p2++;

      // p2 now points to the beginning of the URL
      // The URL is read until a closing ' or ) or whitespace is seen
      p3 = p2+1;
      while (!Character.isWhitespace(sb.charAt(p3)) && sb.charAt(p3) != '\'' && sb.charAt(p3) != ')') p3++;

      StringBuffer link = new StringBuffer();

      // The URL is the text between p2 and p3
      for (int i=p2; i < p3; i++) link.append(sb.charAt(i));

      String index = Integer.toString(app.urls.size());

      if (app.debug) System.out.println("Link " + index + "=" + link.toString());

      // Encode link with random key
      String txt = c5.encode(link);

      link.setLength(0);

      // Save url to vector in parent class
      app.urls.addElement(txt);

      StringBuffer tag = new StringBuffer();

      // insert index as argument for fetch
      for (int i=0; i < p2; i++) tag.append(sb.charAt(i));
      tag.append(index);
      for (int i=p3; i < sb.length(); i++) tag.append(sb.charAt(i));

      return tag.toString();

    }  // end try

    catch (StringIndexOutOfBoundsException e) {
      // Most of the time a StringIndexOutOfBoundsException 
      // here means the tag was not standard conforming so 
      // the algorithm for finding the URL crapped out.
      // If that's the case, the original tag is returned.
    }

    catch (Exception e) {
      System.err.println(e);
    }

    return sb.toString();
  
  } // checkTag

} // fetcher