import java.awt.*;
import java.awt.event.*;
import java.lang.*;

/** navpanel, written by Dan Kiser, 1999
 *  <p>
 *  This class is used to define a navigation panel.
 *  The panel contains buttons used to navigate
 *  through the protected files that have been 
 *  previously loaded.  The following buttons are
 *  displayed by the panel:
 *    Home
 *    Back
 *    Next
 *    Reload
 *    Stop
 */
public class navpanel extends Panel implements ActionListener {

  // Constants

  protected final int FWD = 1;   // Move forward in history list
  protected final int BWD = 2;   // Move backwrd in history list
  protected final int GET = 3;   // Get current URL in history list
  protected final int ADD = 4;   // Add URL to history list

  // Variables

  private Button bhome;          // Home page button
  private Button bnext;          // Next page button
  private Button bprev;          // Prev page button
  private Button bload;          // Reload page button
  private Button bstop;          // Stop button

  private password app;          // Applet reference

  public navpanel(password parent) {

    app = parent;

    GridLayout grid  = new GridLayout(1,5,5,5);

    // Set layout
    setLayout(grid);

    // Home page
    bhome = new Button("Home");
    bhome.addActionListener(this);
    bhome.setEnabled(true);
    add(bhome);

    // Previous page
    bprev = new Button("Back");
    bprev.addActionListener(this);
    bprev.setEnabled(false);
    add(bprev);

    // Next page
    bnext = new Button("Next");
    bnext.addActionListener(this);
    bnext.setEnabled(false);
    add(bnext);

    // Reload page
    bload = new Button("Reload");
    bload.addActionListener(this);
    bload.setEnabled(false);
    add(bload);

    // Stop load
    bstop = new Button("Stop");
    bstop.addActionListener(this);
    bstop.setEnabled(false);
    add(bstop);

    // Set size of panel
    setSize(325,30);

    grid.layoutContainer(this);

  } // navpanel


  public void setEnabled(String name, boolean flag) {

    if (name.equals("Home")) {
      bhome.setEnabled(flag);
    } else if (name.equals("Back")) {
      bprev.setEnabled(flag);
    } else if (name.equals("Next")) {
      bnext.setEnabled(flag);
    } else if (name.equals("Reload")) {
      bload.setEnabled(flag);
    } else if (name.equals("Stop")) {
      bstop.setEnabled(flag);
    }

  } // setEnabled


  public void actionPerformed(ActionEvent e) {
    Button theSource = (Button) e.getSource();

    if (theSource.getLabel().equals("Home")) {
      // Handle home page
      app.home();

    } else if (theSource.getLabel().equals("Back")) {
      // Handle previous page
      app.setList(BWD, null);

    } else if (theSource.getLabel().equals("Next")) {
      // Handle next page
      app.setList(FWD, null);

    } else if (theSource.getLabel().equals("Reload")) {
      // Handle reload page
      app.setList(GET, null);

    } else if (theSource.getLabel().equals("Stop")) {
      // Stop loading page
      app.doc.stop();
    }
      
  } // actionPerformed

} // navpanel
