/** Password applet, written by Dan Kiser, 1999, 2000
 *
 * Copyright (c) 1999, 2000 All Rights Reserved.
 *
 * Permission to use, copy, modify, and distribute this 
 * software and its documentation for NON-COMMERCIAL and
 * COMMERCIAL purposes and without fee is hereby granted 
 * provided that this copyright notice appears in all 
 * copies.
 *
 * THE AUTOR MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT 
 * THE SUITABILITY OF THE SOFTWARE, EITHER EXPRESS OR 
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE IMPLIED 
 * WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR 
 * PURPOSE, OR NON-INFRINGEMENT. THE AUTHOR SHALL NOT BE 
 * LIABLE FOR ANY DAMAGES SUFFERED BY THE USER AS A RESULT 
 * OF USING, MODIFYING OR DISTRIBUTING THIS SOFTWARE OR ITS 
 * DERIVATIVES.
 *
 *  <p>
 *  This applet is used to provide a way to password
 *  protect web pages.  This entire application resides
 *  on the client machine.
 *
 *  After a user enters a valid username and password,
 *  the appet manages the retrieval of all protected
 *  web pages.  It also provides buttons for navigation.
 * 
 *  A page is never cached and cannot be book marked.
 *  If the use_base parameter is set to false, a base
 *  tag will not be inserted into the document.  By
 *  default, JavaScript will still insert a base tag, which
 *  is the base URL of the page containing this applet.
 *  If no base tag is specified, all relative URLs become
 *  relative to this default base tag.
 *  <p>
 *  The applet accepts the following parameter values: 
 *  <ul>
 *  <li>param name=bgcolor value="RGB"       <br>
 *      where RGB is a hex value of 000000 thru ffffff.
 *  <li>param name=home_url value="url"      <br>
 *      where url is the home page url
 *  <li>param name=home_frame value="frame"  <br>
 *      where frame is the home page target frame
 *  <li>param name=error_url value="url"     <br>
 *      where url is the error page url
 *  <li>param name=error_frame value="frame" <br>
 *      where frame is the error page target frame
 *  <li>param name=psw_url value="url"       <br>
 *      where url is the password file url
 *  <li>param name=use_base value="flag"     <br>
 *      where flag is either true or false
 *  <li>param name=query_str value="flag"    <br>
 *      where flag is either true or false
 *  <li>param name=use_nav value="flag"      <br>
 *      where flag is either true or false
 *  </ul>
 *
 */

import java.applet.Applet;
import java.net.URL;
import java.net.MalformedURLException;
import java.lang.*;
import java.io.*;
import java.awt.*;
import java.util.Vector;
import java.util.Hashtable;

public class password extends Applet implements Runnable {

  // Constants

  protected final int FWD = 1;   // Move forward in history list
  protected final int BWD = 2;   // Move backwrd in history list
  protected final int GET = 3;   // Get current URL in history list
  protected final int ADD = 4;   // Add URL to history list
  protected final int MAX = 50;  // Max size of history list

  // Variables

  protected boolean debug=false; // Debugging flag

  protected boolean query=false; // User name in query string flag

  private boolean display=true;  // Display navigation buttons flag 

  private navpanel navpan;       // Navigation panel

  private URL homeURL=null;      // Home page URL

  private URL errorURL=null;     // Error page URL

  private URL pswURL=null;       // Password file URL

  private String hframe;         // Home page target frame

  private String eframe;         // Error page target frame

  protected boolean base=true;   // Use base tag flag

  protected Vector urls = new Vector();        // List of URLs

  private pswframe pswWin;       // Password entry window

  private Thread pswMon;         // Thread to monitor password entry window

  private Vector list = new Vector(10);        // History list

  private int curr=-1;           // Current page index

  private boolean load=false;    // Load page flag

  protected StringBuffer content = new StringBuffer(); // Holds page content

  protected fetcher doc;         // Holds a fetcher object

  protected String user="";      // Holds user name

  protected String surl;         // Holds starting URL

  private boolean ready=false;   // Indicates Script has user name

  protected String hostIP;       // Holds IP address of applet host



  //*******************************************************
  // This method initializes the applet.  It creates the
  // navigation panel and fetches the parameter values.
  //
  public void init() {

    String attr;

    System.out.println("Password applet by Dan Kiser, v2.5");  

    // Get background color
    attr = getParameter("bgcolor");
    int bgcolor;
    try {
      bgcolor = (attr == null) ? Color.white.getRGB() : (Integer.parseInt(attr,16));
    }
    catch (Exception e) { bgcolor = Color.white.getRGB(); }

    setBackground(new Color(bgcolor));

    // Get home page URL
    attr = getParameter("home_url");
    if (attr != null) homeURL = getURL(attr);

    // Get home page target fame
    attr = getParameter("home_frame");
    if (attr != null) {
      hframe = new String(attr);
    }
    else {
      hframe = new String("_top");
    }

    // Get error page URL
    attr = getParameter("error_url");
    if (attr != null) {
      errorURL = getURL(attr);
    }
    else {
      errorURL = getURL("./error.htm");
    }

    // Get error page target fame
    attr = getParameter("error_frame");
    if (attr != null) {
      eframe = new String(attr);
    }
    else {
      eframe = new String("_top");
    }

    // Get password file URL
    attr = getParameter("psw_url");
    if (attr != null)  {
      pswURL = getURL(attr);
    }
    else {
      pswURL = getURL("./users.txt");
    }

    // Get base tag flag
    attr = getParameter("use_base");
    if (attr != null) {
      base = (attr.equalsIgnoreCase("false")) ? false : true;
    }

    // Get query string flag
    attr = getParameter("query_str");
    if (attr != null) {
      query = (attr.equalsIgnoreCase("false")) ? false : true;
    }

    // Get navigation flag
    attr = getParameter("use_nav");
    if (attr != null) {
      display = (attr.equalsIgnoreCase("false")) ? false : true;
    }       

    // Get debug flag
    attr = getParameter("debug");
    if (attr != null) {
      debug = (attr.equalsIgnoreCase("false")) ? false : true;
      if (debug) System.out.println("Trace debugging enabled...");  
    }       

    // Set font
    setFont(new Font("SansSerif", Font.PLAIN, 14));

    if (display) {
      // Create navpanel
      navpan = new navpanel(this);
      navpan.setVisible(false);
      add(navpan);
    }

  } // init


  //*******************************************************
  // This method creates a valid URL object for a file name.
  //
  protected URL getURL(String filename) {
    URL docBase = this.getDocumentBase();
    URL url = null;

    try {
      if (filename.indexOf(":") == -1) {
        url = new URL(docBase, filename);
      }
      else {
        url = new URL(filename);
      }
    } catch (MalformedURLException e) {
      System.err.println("Couldn't create URL: " + e);
      return null;
    }

    return url;
  }


  //*******************************************************
  // This method opens the login window.
  //
  public void start() {

    if (debug) System.out.println("Starting applet...");

    if (pswMon == null) {
      pswWin = new pswframe(this, pswURL);
      pswMon = new Thread(this);
      pswMon.start();
    }

  } // start


  //*******************************************************
  // This method handles the login procedure.
  //
  public void run() {

    if (debug) System.out.println("Entering monitor thread...");

    int i = 0;

    // Display login box for one minute.
    while (pswWin.isShowing() && i < 120) {
      try {
        pswMon.sleep(500);
        i++;
      }
      catch (InterruptedException e) { 
        System.err.println(e); 
      }
    }

    if (pswWin.isValid()) {
      surl = pswWin.getStartURL();
      user = pswWin.getUser();
      pswWin.dispose();

      if (display) {
        navpan.setVisible(true);
        if (debug) System.out.println("Load starting URL");
        history hist = new history(surl);
        setList(ADD, hist); // Save URL on history list
        setList(GET, null); // Force start page to load
      }
      else {
        while(!ready) {
          try {
            pswMon.sleep(500);
          }
          catch (InterruptedException e) { 
            System.err.println(e); 
          }
        }
        enter();
      }
    }
    else {
      pswWin.dispose();
      if (pswMon != null) {
        redirect(errorURL, eframe);
      }
    }

    if (debug) System.out.println("Exiting monitor thread...");

  } // run


  //*******************************************************
  // This method stops the applet.
  //
  public void stop() {

    if (debug) System.out.println("Stopping monitor thread...");

    pswMon = null;

    pswWin.dispose();

  } // stop


  //*******************************************************
  // This method is used by JavaScript to set a flag that
  // signals the applet to redirect the browser to the
  // start page.  The call should be made JavaScript gets
  // the user name and stores it in a cookie.
  //
  public synchronized void setReady() {

    ready = true;

  } // setReady


  //*******************************************************
  // This method is used by JavaScript to retrieve the
  // content of an URL when the user clicks on a navigation
  // button.
  //
  public synchronized String getPage() {

    content.setLength(0);

    if (load) {

      if (debug) System.out.println("Loading page #" + curr);

      navpan.setEnabled("Reload",false);
      navpan.setEnabled("Stop",true);

      history hist = (history)list.elementAt(curr);

      if (!hist.hasData()) {
        doc = new loader(this,hist.getURL());
      }
      else {
        // Only post form data if it has not yet been posted.
        if (hist.isNew(false)) {
          doc = new poster(this,hist.getURL(),hist.getData());
        }
        else {
          doc = new fetcher();
          content.append("<html><body><hr><h3>This page cannot be displayed because it was created by a form POST request.<br>Click the \"Reload\" button if you wish to POST again.</h3></body></html>");
        }
      }
      doc.start();

      try {
        doc.join();  // Wait for thread to retrieve page
      }
      catch (InterruptedException e) { 
        content.append("<hr><h3>Transfer interrupted</h3><br>" + e);
      }

      if (content.length() == 0) {
        content.append("<html><body><hr><h3>Document not loaded and/or not found</h3></body></html>");
      }

      navpan.setEnabled("Stop",false);
      navpan.setEnabled("Reload",true);

      load = false;

      if (debug) System.out.println("Returning page #" + curr);

    }

    System.gc();

    return content.toString();

  } // getPage


  //*******************************************************
  // This method is used to manage an URL history list.  
  // The list pointer is adjusted and the list navigation 
  // buttons are either enabled or disabled. 
  // 
  public synchronized void setList(int action, history theURL) {

    if (debug) System.out.println("Action="+action);

    // Load prev page
    if (action == BWD) {

      if (curr > 0) {
        curr--;
        load = true;
      }

    }

    // Load next page
    else if (action == FWD) {

      if (curr < list.size()-1) {
        curr++;
        load = true;
      }

    } 

    // Reload current page
    else if (action == GET) {
      load = true;
      // Check if reposting form data.
      history hist = (history)list.elementAt(curr);
      if (hist.hasData()) hist.isNew(true);
    }

    // Add page to list
    else if (action == ADD) {

      if (curr == list.size()-1) {

        if (list.size() < MAX) {
          list.addElement(theURL);
          curr++;
        }
        else {
          list.removeElementAt(0);
          list.addElement(theURL);
        }

      } 
      else {
        list.insertElementAt(theURL,curr+1);
        curr++;

        for(int i=list.size()-1; i > curr; i--) {
          list.removeElementAt(i);
        }

      }

    }

    // Set prev and next buttons
    if (curr > 0) {
      navpan.setEnabled("Back",true);
    }
    else {
      navpan.setEnabled("Back",false);
    }

    if (curr < list.size()-1) {
      navpan.setEnabled("Next",true);
    }
    else {
      navpan.setEnabled("Next",false);
    }

  } // setList


  //*******************************************************
  // This method is called from JavaScript when a hyperlink 
  // to a secret page is clicked.  The input includes the 
  // index for the link located in the URL vector.
  //
  public void fetchURL(String index) {

    if (debug) System.out.println("Fetch link " + index + "...");

    int i = Integer.valueOf(index).intValue();

    String url = (String)urls.elementAt(i);

    history hist = new history(url);

    setList(ADD, hist);  // Save URL on history list
    setList(GET, null);  // Force page to load

  } // fetchURL


  //*******************************************************
  // This method is called from JavaScript when a form's 
  // submit button is clicked to post data to a secret link.
  // The input includes the index for the link located in 
  // the URL vector and form data as a string of name/value 
  // pairs (name1=value1&name2=value2& ... nameN=valueN)
  //
  public void fetchURL(String index, String data) {

    if (debug) System.out.println("Fetch link " + index + "...");

    int i = Integer.valueOf(index).intValue();

    String url = (String)urls.elementAt(i);

    history hist = new history(url, data);

    setList(ADD, hist);  // Save URL on history list
    setList(GET, null);  // Force page to load

  } // fetchURL


  //*******************************************************
  // This method is used by JavaScript to retrieve the
  // user name.
  //
  public synchronized String getUser() {

    return user;

  } // getUser


  //*******************************************************
  // This method is used to redirect the browser to the
  // starting page after the user is validated.  The param
  // use_nav must be set to false for this method to be
  // called.
  //
  private void enter() {

    URL theURL;

    cipher5 c5 = new cipher5(this);

    StringBuffer sbUrl = c5.decode(surl);

    // Check to see if user name should be appended to 
    // query string.
    boolean isQuery = false;

    for (int i=0; i < sbUrl.length(); i++) {
      if (sbUrl.charAt(i) == '?') isQuery = true;
    }
  
    if (isQuery) {
      sbUrl.append("&user=");
      sbUrl.append(user);
    }
    else {
      sbUrl.append("?user=");
      sbUrl.append(user);
    }

    // Check if URL is relative or absolute.
    boolean isAbsolute = false;

    for (int i=0; i < sbUrl.length(); i++) {
      if (sbUrl.charAt(i) == ':') isAbsolute = true;
    }

    try {
      if (!isAbsolute) {
        theURL = new URL(this.getDocumentBase(), sbUrl.toString());
      }
      else {
        theURL = new URL(sbUrl.toString());
      }

      redirect(theURL, "_self");
    }
    catch (MalformedURLException e) { System.err.println(e); }  

  } // enter


  //*******************************************************
  // This method is used to redirect the browser to the
  // Home page.
  //
  public void home() {

    if (homeURL != null) {
      redirect(homeURL, hframe);
    }
    else {
      history hist = new history(surl);

      setList(ADD, hist);  // Save URL on history list
      setList(GET, null);  // Force page to load
    }

  } // home


  //*******************************************************
  // This method is used to redirect the browser to the
  // specified URL and display it in the destination
  // window.
  //
  private void redirect(URL theURL, String dest) {

    if (debug) System.out.println("Redirect URL=" + theURL + " Frame="+ dest);

    try {
      InputStream in = theURL.openStream();
      int input = in.read();
      in.close();
      if (dest.length() > 0) {
        getAppletContext().showDocument(theURL, dest);
      }
      else {
        getAppletContext().showDocument(theURL);
      }
    }
    catch (Exception e) { System.err.println(e); }

  } // redirect

} // password 
